<?php  
defined('BASEPATH') or exit('No direct script access allowed');

class Neropay_gateway extends App_gateway
{
    public function __construct()
    {
        // Parent constructor çağrılıyor.
        parent::__construct();

        /**
         * Gateway unique id - REQUIRED
         * Bu ID, dosya ismi (Neropay_gateway.php) ve sınıf ismi içinde yer almalıdır.
         */
        $this->setId('neropay_gateway');

        /**
         * REQUIRED - Gateway adı
         */
        $this->setName('NeroPAY Payment Gateway');

        /**
         * Gateway ayarlarını tanımlıyoruz.
         * Burada ayar isimleri public_api_key ve secret_api_key olarak belirlenmiştir.
         */
        $this->setSettings(array(
            array(
                'name'      => 'public_api_key',  // Ayarın ismi: public_api_key
                'label'     => 'Public API Key',
                'type'      => 'input',
            ),
            array(
                'name'  => 'secret_api_key',  // Ayarın ismi: secret_api_key
                'label' => 'Secret API Key',
                'type'  => 'input'
            ),
            array(
                'name'          => 'currencies',
                'label'         => 'Supported Currencies',
                'default_value' => 'GBP,EUR,SEK,CHF,PLN,IDR,DKK,CZK,BGN,NOK',
            ),
        ));
    }

    /**
     * Müşteri faturadaki "Pay Now" butonuna tıkladığında çağrılır.
     * Ödeme başlatma isteğini hazırlar ve kullanıcıyı NeroPAY API’ye yönlendirir.
     *
     * @param array $data Fatura, ödeme tutarı ve müşteri bilgilerini içerir.
     * @return mixed
     */
    public function process_payment($data)
    {
        // Fatura bilgisi kontrol ediliyor
        if (!isset($data['invoice'])) {
            echo "Error: Invoice data is missing.";
            exit;
        }

        $invoice = $data['invoice'];
        
        // Sonrasında iptal veya başarılı metodlarında kullanılmak üzere oturum bilgisi kaydediliyor
        $_SESSION['invoice_id'] = $invoice->id;
        $_SESSION['url']        = $_SERVER['HTTP_REFERER'];

        // Ödeme açıklaması oluşturuluyor
        $description = "Invoice Payment for #" . (isset($invoice->number) ? $invoice->number : $invoice->id);
        if (empty($description)) {
            echo "Error: Payment description is required.";
            exit;
        }

        // Ayarlar üzerinden Public API Key çekiliyor (yeni ayar ismi kullanılıyor)
        $publicKey = get_option('paymentmethod_neropay_gateway_public_api_key');
        if (empty($publicKey)) {
            echo "Error: Public API key is not configured.";
            exit;
        }

        // Faturada belirtilen para birimi, yoksa varsayılan olarak GBP
        $currency = !empty($invoice->currency_name) ? $invoice->currency_name : 'GBP';

        // --- Para Birimi Kontrolü Başlangıç ---
        // Ayarlardan desteklenen para birimlerini alıyoruz (virgülle ayrılmış liste)
        $supportedCurrencies = get_option('currencies');
        if (empty($supportedCurrencies)) {
            $supportedCurrencies = 'GBP';
        }
        // Boşlukları temizleyip diziye dönüştürüyoruz
        $supportedCurrencies = explode(',', str_replace(' ', '', $supportedCurrencies));
        // Fatura para biriminin desteklenip desteklenmediğini kontrol ediyoruz
        if (!in_array($currency, $supportedCurrencies)) {
            echo "Error: Currency not supported: " . $currency;
            exit;
        }
        // --- Para Birimi Kontrolü Bitiş ---

        // Ödeme tutarı (fatura toplamı veya $data['amount'])
        $amount = !empty($invoice->total) ? $invoice->total : $data['amount'];

        // Müşteri bilgileri alınıyor
        if (!isset($invoice->client)) {
            echo "Error: Client information is missing from invoice.";
            exit;
        }
        
        $client = $invoice->client;

        // Müşteri adı oluşturuluyor
        $customer_name = '';
        if (!empty($client->company)) {
            $customer_name = $client->company;
        } elseif (!empty($client->firstname) && !empty($client->lastname)) {
            $customer_name = $client->firstname . ' ' . $client->lastname;
        }
        if (empty($customer_name)) {
            echo "Error: Customer name is required.";
            exit;
        }

        // Müşteri e-posta bilgisi alınıyor
        $customer_email = !empty($client->email) ? $client->email : null;

        // E-posta bilgisi yoksa, veritabanından çekmeye çalışıyoruz
        if (!$customer_email && isset($invoice->clientid)) {
            $ci =& get_instance();
            $ci->load->model('clients_model');
            $client_details = $ci->clients_model->get($invoice->clientid);
            if (!empty($client_details) && !empty($client_details->email)) {
                $customer_email = $client_details->email;
            }
        }

        // Yine bulunamazsa varsayılan e-posta adresi kullanılıyor
        if (!$customer_email) {
            $customer_email = 'perfex@neropay.app';
        }

        // İsteğe bağlı olarak telefon ve adres bilgileri alınıyor
        $customer_phone   = !empty($client->phonenumber) ? $client->phonenumber : '';
        $customer_address = !empty($client->address) ? $client->address : '';

        // NeroPAY API isteği için parametreler hazırlanıyor
        $params = [
            'identifier'       => uniqid(), // Benzersiz işlem ID'si
            'currency'         => $currency,
            'amount'           => $amount,
            'details'          => $description,
            'ipn_url'          => site_url('neropay_gateway/ipn_listener'),
            'cancel_url'       => site_url('invoice/' . $invoice->id . '/' . $invoice->hash),
            'success_url'      => site_url('invoice/' . $invoice->id . '/' . $invoice->hash),
            'public_key'       => $publicKey,
            'customer_name'    => $customer_name,
            'customer_email'   => $customer_email,
            'customer_phone'   => $customer_phone,
            'customer_address' => $customer_address,
        ];

        // NeroPAY API endpoint URL
        $endpoint = 'https://manage.neropay.co.uk/payment/initiate';

        // cURL ile API isteği gönderiliyor
        $ch = curl_init($endpoint);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_POSTFIELDS, $params);

        $response = curl_exec($ch);

        // cURL hatası kontrolü
        if (curl_errno($ch)) {
            echo "cURL error: " . curl_error($ch);
            curl_close($ch);
            exit;
        }

        curl_close($ch);

        // API yanıtı JSON olarak decode ediliyor
        $result = json_decode($response, true);
        if ($result === null) {
            echo "JSON decode error. Response: " . $response;
            exit;
        }

        // Eğer ödeme başlatıldıysa kullanıcı ilgili URL’ye yönlendiriliyor
        if (isset($result['success']) && $result['success'] == 'ok') {
            header('Location: ' . $result['url']);
            exit;
        } else {
            // API'den gelen raw yanıtı doğrudan ekrana bastırıyoruz
            echo "API Response: " . $response;
        }
    }

    /**
     * NeroPAY'den gelen Instant Payment Notification (IPN) mesajlarını dinler.
     * Ödeme durumu kontrol edilip loglanır.
     */
    public function ipn_listener()
    {
        // Gateway tarafından gönderilen POST parametreleri alınıyor
        $status     = $_POST['status'];
        $signature  = $_POST['signature'];
        $identifier = $_POST['identifier'];
        $data       = $_POST['data'];

        // Secret API key, yeni ayar ismi kullanılarak çekiliyor
        $secret = get_option('paymentmethod_neropay_gateway_secret_api_key');

        // Gönderilen verilerle imza oluşturuluyor
        $customKey   = $data['amount'] . $identifier;
        $mySignature = strtoupper(hash_hmac('sha256', $customKey, $secret));

        // Ödeme başarılı ise ve imzalar eşleşiyorsa, log kaydı oluşturuluyor
        if ($status == 'success' && $signature == $mySignature) {
            log_activity('NeroPAY Payment Successful: ' . json_encode($data));
            // Burada fatura durumunu güncelleme gibi işlemler yapılabilir.
        } else {
            log_activity('NeroPAY Payment Failed: ' . json_encode($data));
            // Hata durumunda gerekli işlemler burada yapılır.
        }
    }
    
    /**
     * Ödeme iptal edildiğinde çağrılır.
     * Kullanıcı faturanın detay sayfasına yönlendirilir.
     */
    public function cancel()
    {
        $invoice_id = $_SESSION['invoice_id'];
        set_alert('warning', 'Payment has been cancelled.');
        redirect(site_url('invoices/view/' . $invoice_id));
    }
    
    /**
     * Ödeme başarılı olduğunda çağrılır.
     * Kullanıcı faturanın detay sayfasına yönlendirilir.
     */
    public function success()
    {
        $invoice_id = $_SESSION['invoice_id'];
        set_alert('success', 'Payment was successful.');
        redirect(site_url('invoices/view/' . $invoice_id));
    }
}
?>
